@@ -15,11 +15,12 @@ from mash import io_util
 from mash.filesystem.filesystem import FileSystem, cd
 from mash.io_util import log, shell_ready_signal, print_shell_ready_signal, check_output
 from mash.shell import delimiters
-from mash.shell.delimiters import comparators, DEFINE_FUNCTION, FALSE, IF, LEFT_ASSIGNMENT, RETURN, RIGHT_ASSIGNMENT, THEN, TRUE
+from mash.shell.if_statement import LINE_INDENT, Abort, Done, handle_else_statement, handle_if_statement, handle_then_else_statements, handle_then_statement
+from mash.shell.delimiters import ELSE, comparators, DEFINE_FUNCTION, FALSE, IF, LEFT_ASSIGNMENT, RETURN, RIGHT_ASSIGNMENT, THEN, TRUE
 from mash.filesystem.scope import Scope, show
 from mash.shell.errors import ShellError, ShellPipeError
 from mash.shell.function import InlineFunction
-from mash.shell.parsing import expand_variables, expand_variables_inline, filter_comments, infer_infix_args, parse_commands, quote_items
+from mash.shell.parsing import expand_variables, expand_variables_inline, filter_comments, indent_with, infer_infix_args, inline_indent_with, parse_commands, quote_items
 from mash.util import for_any, has_method, identity, is_valid_method_name, omit_prefixes, removeprefix, split_prefixes, translate_terms
 
 
@@ -30,6 +31,7 @@ COMMENT = '#'
 LAST_RESULTS = '_last_results'
 LAST_RESULTS_INDEX = '_last_results_index'
 INNER_SCOPE = 'inner_scope'
+RAW_LINE_INDENT = 'raw_line_indent'
 ENV = 'env'
 
 Command = Callable[[Cmd, str], str]
@@ -109,6 +111,10 @@ class BaseShell(Cmd):
         items.remove('#')
         return items
 
+    @property
+    def _last_if(self):
+        return self.locals[IF][-1]
+
     @property
     def _last_results(self):
         return self.env[LAST_RESULTS]
@@ -446,6 +452,7 @@ class BaseShell(Cmd):
 
         try:
             line = self.onecmd_prehook(line)
+            self.locals.set(RAW_LINE_INDENT, indent_with(line))
 
             if DEFINE_FUNCTION in self.locals and self.locals[DEFINE_FUNCTION].multiline:
                 self._define_multiline_function(line)
@@ -456,7 +463,8 @@ class BaseShell(Cmd):
                 result = self.run_commands(lines)
 
                 if print_result and result is not None:
-                    print(result)
+                    if result or not self.locals[IF]:
+                        print(result)
 
         except CancelledError:
             pass
@@ -516,11 +524,13 @@ class BaseShell(Cmd):
         if not lines:
             return
 
-        self.locals.set(IF, [])
         if LEFT_ASSIGNMENT in self.locals:
             self.locals.rm(LEFT_ASSIGNMENT)
 
         for i, line in enumerate(lines):
+            # indent = inline_indent_with(*self.locals[RAW_LINE_INDENT], i)
+            indent = self.locals[RAW_LINE_INDENT] + (i,)
+            self.locals.set(LINE_INDENT, indent)
 
             if DEFINE_FUNCTION in self.locals:
                 self._extend_inline_function_definition(line)
@@ -571,6 +581,10 @@ class BaseShell(Cmd):
             self._save_assignee(result)
             return
 
+        if RETURN in self.locals:
+            self.locals.set(RETURN,  result)
+            return
+
         return result
 
     def _conditionally_insert_assign_operator(self, lines, i, line):
@@ -599,29 +613,69 @@ class BaseShell(Cmd):
         prefixes, line, infix_operator_args = self.parse_single_command(
             command_and_args)
 
+        # TODO refactor
+        # TODO do this recursively
+        for _ in range(4):
+            # TODO handle case of both THEN and ELSE in prefixes
+            if not self.locals[IF]:
+                break
+            elif IF in prefixes:
+                if self.locals[LINE_INDENT] <= self._last_if[LINE_INDENT]:
+                    # case of: new, independent if-then
+                    self.locals[IF].pop()
+                else:
+                    break
+            elif THEN not in prefixes:
+                if ELSE in prefixes:
+                    if self.locals[LINE_INDENT][:2] < self._last_if[LINE_INDENT][:2]:
+                        # case of: unclosed if-then-else
+                        self.locals[IF].pop()
+                    else:
+                        break
+                # elif self.locals[LINE_INDENT][:2] <= self._last_if[LINE_INDENT][:2]:
+                elif self.locals[LINE_INDENT] <= self._last_if[LINE_INDENT]:
+                    # case of: if-then fully terminated
+                    self.locals[IF].pop()
+                else:
+                    break
+            else:
+                break
+
+        if self.locals[IF] and IF not in prefixes and THEN not in prefixes and ELSE not in prefixes:
+            c = for_any([IF, THEN, ELSE], contains, prefixes)
+            assert not c, c
+            try:
+                # TODO don't modify locals[IF]
+                if self._last_if['branch'] == THEN or self._last_if['branch'] is None:
+                    handle_then_statement(self, transparent=True)
+                elif self._last_if['branch'] == ELSE:
+                    # TODO don't drop from locals[IF]
+                    handle_else_statement(self, transparent=True)
+
+            except Abort as result:
+                return prev_result
+
         if prefixes:
-            if THEN in prefixes:
-                if not self.locals[IF]:
-                    if self.ignore_invalid_syntax:
-                        return ''
-                    raise ShellError(
-                        f'If-then clause requires an {IF} statement')
-
-                self.locals[IF][-1]['depth'] += 1
-
-                if len(prefixes) <= 1 and self.locals[IF][-1]['depth'] >= 2:
-                    raise ShellError(
-                        f'If-then clause requires an {IF} statement')
-
-                if not self.locals[IF][-1]['value']:
-                    # skip
-                    return ''
-                # otherwise continue
+            if THEN in prefixes or ELSE in prefixes:
+                try:
+                    handle_then_else_statements(self, prefixes, prev_result)
+                except Done as result:
+                    return result.args[0]
 
                 if not self.is_function(line.split(' ')[0]):
                     line = 'echo ' + line
 
-            if prefixes[-1] in delimiters.bash:
+            if RETURN in prefixes:
+                if RETURN not in self.locals:
+                    self.locals.set(RETURN, None)
+
+                if len(prefixes) == 1 and not self.is_function(line.split(' ')[0]):
+                    line = 'echo ' + line
+
+            if prefixes[-1] == IF:
+                return handle_if_statement(self, line, prev_result)
+
+            elif prefixes[-1] in delimiters.bash:
                 return self.pipe_cmd_sh(line, prev_result, delimiter=prefixes[-1])
 
             elif prefixes[-1] == '>>=':
@@ -636,22 +690,6 @@ class BaseShell(Cmd):
                 self.set_env_variables(line, prev_result)
                 return ''
 
-            elif prefixes[-1] == IF:
-                assert_if_then_is_closed = False
-                if assert_if_then_is_closed:
-                    if self.locals[IF] and self.locals[IF][-1]['depth'] == 0:
-                        raise ShellError(
-                            f'The previous if-then statement was not closed')
-
-                if self.locals[IF] and not self.locals[IF][-1]['value']:
-                    # force value to be false when previous conditions are false
-                    value = False
-                else:
-                    value = self.eval_compare(line)
-
-                self.locals[IF].append({'value': value, 'depth': 0})
-                return ''
-
         if infix_operator_args:
             return self.infix_command(*infix_operator_args)
         elif is_function_definition(command_and_args):
@@ -906,6 +944,11 @@ class BaseShell(Cmd):
             for line in f.inner:
                 self.onecmd(line, print_result=False)
 
+                if RETURN in self.locals:
+                    if self.locals[RETURN] is None:
+                        raise ShellError('invalid state')
+                    return self.locals[RETURN]
+
             terms = [term for term in f.command.split(' ') if term != '']
             if not f.multiline:
                 terms = list(translate_terms(terms, translations))
@@ -931,17 +974,31 @@ class BaseShell(Cmd):
             args[-1] = args[-1][:-1]
 
         self.locals.set(DEFINE_FUNCTION,
-                        InlineFunction('', *args, func_name=f))
+                        InlineFunction('', *args, func_name=f,
+                                       line_indent=self.locals[RAW_LINE_INDENT]))
 
-    def _define_multiline_function(self, line: str):
-        line = line.lstrip()
-        if line.startswith(RETURN + ' '):
+    def _define_multiline_function(self, indented_line: str):
+        line = indented_line.lstrip()
+
+        if line.startswith(RETURN + ' ') and (
+                self.locals[RAW_LINE_INDENT] <= self.locals[DEFINE_FUNCTION].line_indent or
+                not self.locals[DEFINE_FUNCTION].inner):
+
+            if self.locals[RAW_LINE_INDENT] < self.locals[DEFINE_FUNCTION].line_indent:
+                raise ShellError(
+                    f'Function defintion did not end with {RETURN}')
+
+            # TODO fix indent
             line = removeprefix(line, RETURN + ' ')
             self.locals[DEFINE_FUNCTION].command = line
             self._save_inline_function()
 
         else:
-            self.locals[DEFINE_FUNCTION].inner.append(line)
+            # update line indent on the first line
+            if self.locals[DEFINE_FUNCTION].inner == []:
+                self.locals[DEFINE_FUNCTION].line_indent = self.locals[RAW_LINE_INDENT]
+
+            self.locals[DEFINE_FUNCTION].inner.append(indented_line)
 
     def _save_inline_function(self) -> str:
         func = self.locals[DEFINE_FUNCTION]
