@@ -18,9 +18,9 @@ class Python(Enum):
     RIGHT_ASSIGNMENT = '->'
     PIPE = '|>'
     MAP = '>>='
+    ELSE = 'else'
     SET_ENV_VARIABLE = '='
     INLINE_COMMENT = '#'
-    # ELSE = 'else'
     # AND = 'and'
     # OR = 'or'
 
@@ -36,7 +36,7 @@ RIGHT_ASSIGNMENT = Python.RIGHT_ASSIGNMENT.value
 LEFT_ASSIGNMENT = Python.LEFT_ASSIGNMENT.value
 IF = Python.IF.value
 THEN = Python.THEN.value
-# ELSE = Python.ELSE.value
+ELSE = Python.ELSE.value
 # AND = Python.AND.value
 # OR = Python.OR.value
 
