@@ -57,9 +57,10 @@ class ShellFunction:
 
 # TODO rename ShellFunction to ShellMethod, and rename InlineFunction to Function
 class InlineFunction:
-    def __init__(self, command: str, *args: str, func_name='', multiline=False) -> None:
+    def __init__(self, command: str, *args: str, func_name='', multiline=False, line_indent=None) -> None:
         self.command = command
         self.args = args
         self.func_name = func_name
         self.multiline = multiline
+        self.line_indent = line_indent
         self.inner = []
